"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var babel_preset_react_app_1 = __importDefault(require("babel-preset-react-app"));
var removeFlowPreset = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    // Pass along arguments to babel-preset-react-app and generate its preset.
    var preset = babel_preset_react_app_1.default.apply(void 0, args);
    // Replace the Flow preset with TypeScript.
    preset.presets = preset.presets.filter(function (p) { return !babelPluginTransformFlowDetectionHack(p); });
    preset.presets.push("@babel/preset-typescript");
    return preset;
};
exports.default = removeFlowPreset;
/**
 * Different versions of Babel 7 behave differently when it comes to supplying
 * presets. Later versions wrap imported presets in a function that performs
 * a Babel version check. As a result, we can not do a string comparison to
 * determine if the supplied preset is the Flow plugin using the function name.
 *
 * Later versions of babel-preset-react-app have an option to disable the Flow
 * plugin. To remain compatible with older react-scripts 2 beta versions, we do
 * not use that feature. When react-scripts 2 is officially released, this hack
 * should be removed.
 *
 * @see https://github.com/facebook/create-react-app/tree/next/packages/babel-preset-react-app#usage-with-typescript
 * @see https://github.com/babel/babel/tree/master/packages/babel-plugin-transform-flow-strip-types
 */
var babelPluginTransformFlowDetectionHack = function (preset) {
    /* tslint:disable-next-line:no-var-requires */
    var babel = require("@babel/core");
    try {
        var transformedCode = babel.transform("function foo(one: any, two: number, three?): string {}", {
            presets: [preset]
        });
        return !transformedCode.code.includes("string");
    }
    catch (_a) {
        return false;
    }
};
